import React, { useEffect, useState } from 'react';
import { config, getDataSourceSrv } from '@grafana/runtime';
import { DataQuery, DataSourceInstanceSettings, GrafanaTheme2 } from '@grafana/data';
import { css } from '@emotion/css';
import { Button, Modal, useStyles2 } from '@grafana/ui';
import { getRecordedQueriesAsync } from './state/actions';
import { connect, ConnectedProps } from 'react-redux';
import { getRecordedQueryItems } from './state/selectors';
import { EnterpriseStoreState, RecordedQuery } from '../types';
import { MIXED_DATASOURCE_NAME } from 'app/plugins/datasource/mixed/MixedDataSource';
import { DataSourceFilter } from './DatasourceFilter';
import { QueryCard } from './QueryCard';

function mapStateToProps(state: EnterpriseStoreState) {
  return {
    recordedQueries: getRecordedQueryItems(state.recordedQueries),
  };
}

interface OwnProps<TQuery extends DataQuery> {
  query?: TQuery;
  queries?: Array<Partial<TQuery>>;
  onAddQuery?: (query: TQuery) => void;
  onChangeDataSource?: (ds: DataSourceInstanceSettings) => void;
  dataSource?: DataSourceInstanceSettings;
  filterValue?: DataSourceSettings;
}

const mapDispatchToProps = {
  getRecordedQueriesAsync,
};

const connector = connect(mapStateToProps, mapDispatchToProps);

export type Props<TQuery extends DataQuery> = OwnProps<TQuery> & ConnectedProps<typeof connector>;

export const RecordedQueryAddModalUnconnected = <TQuery extends DataQuery>({
  onAddQuery,
  getRecordedQueriesAsync,
  recordedQueries,
  onChangeDataSource,
  filterValue,
}: Props<TQuery>) => {
  const [isOpen, setIsOpen] = useState(false);
  const [selectedDataSource, setSelectedDataSource] = useState<DataSourceSettings | undefined>(filterValue);
  const styles = useStyles2(getStyles);

  useEffect(getRecordedQueriesAsync, [isOpen, getRecordedQueriesAsync]);

  const showRecordQuery = config?.licenseInfo?.hasLicense && config?.featureToggles?.recordedQueries;
  if (!showRecordQuery || !onChangeDataSource || !onAddQuery) {
    return null;
  }

  const queriesToDisplay = recordedQueries
    .filter((rq: RecordedQuery) => rq.active)
    .filter((rq: RecordedQuery) => !selectedDataSource || rq.queries[0].datasourceId === selectedDataSource.id)
    .map((rq) => (
      <AddQueryCard
        key={rq.id}
        onAddQuery={onAddQuery}
        onChangeDataSource={onChangeDataSource}
        setIsOpen={setIsOpen}
        recordedQuery={rq}
      />
    ));

  return (
    <>
      <Button
        icon="plus"
        onClick={() => {
          setIsOpen(true);
        }}
        variant="secondary"
        className={styles.button}
      >
        Recorded Query
      </Button>
      <Modal isOpen={isOpen} title={'Add Recorded Query'} onDismiss={() => setIsOpen(false)} className={styles.modal}>
        <DataSourceFilter
          value={selectedDataSource?.name}
          onChange={(ds: any) => {
            setSelectedDataSource(ds);
          }}
        />
        {queriesToDisplay.length > 0 ? (
          queriesToDisplay
        ) : (
          <div>No recorded queries available for {selectedDataSource?.name}</div>
        )}
      </Modal>
    </>
  );
};

interface AddProps<TQuery extends DataQuery> {
  onAddQuery: (query: TQuery) => void;
  onChangeDataSource: (ds: DataSourceInstanceSettings) => void;
  setIsOpen: (o: boolean) => void;
  recordedQuery: RecordedQuery;
}

const AddQueryCard = <TQuery extends DataQuery>({
  onAddQuery,
  recordedQuery,
  onChangeDataSource,
  setIsOpen,
}: AddProps<TQuery>) => {
  const addRecordedQuery = async () => {
    const mixedDs = await getDataSourceSrv().getInstanceSettings(MIXED_DATASOURCE_NAME);
    await onChangeDataSource(mixedDs!);

    onAddQuery(({
      expr: `${recordedQuery.name}{id="${recordedQuery.id}"}`,
      legendFormat: '',
      interval: '',
      exemplar: true,
      datasource: recordedQuery.dest_data_source_uid,
    } as unknown) as TQuery);
    setIsOpen(false);
  };

  return (
    <QueryCard
      key={recordedQuery.id}
      recordedQuery={recordedQuery}
      buttons={[
        <Button key={`${recordedQuery.id}-add`} onClick={addRecordedQuery}>
          Add
        </Button>,
      ]}
    />
  );
};

interface DataSourceSettings {
  name: string;
  id: number;
}

const getStyles = (theme: GrafanaTheme2) => {
  return {
    button: css`
      margin-left: -8px;
    `,
    modal: css`
      width: 750px;
    `,
  };
};

export const RecordedQueryAddModal = connector(RecordedQueryAddModalUnconnected);
