import React, { useState } from 'react';
import { css } from '@emotion/css';
import { config, getDataSourceSrv } from '@grafana/runtime';
import { Alert, AlertVariant, Button, Field, Form, Icon, Input, Modal, Switch } from '@grafana/ui';
import { AppEvents, DataQuery, DataSourceInstanceSettings } from '@grafana/data';
import { QueryOperationAction } from 'app/core/components/QueryOperationRow/QueryOperationAction';
import { IdDataQuery, RecordedQuery } from '../types/recordedQuery';
import { saveRecordedQuery, testRecordedQuery } from './state/actions';
import { describeInterval, intervalToSeconds } from '@grafana/data/src/datetime/rangeutil';
import appEvents from 'app/core/app_events';

interface Props {
  query?: DataQuery;
  queries?: Array<Partial<DataQuery>>;
  onAddQuery?: (query: DataQuery) => void;
  onChangeDataSource?: (ds: DataSourceInstanceSettings) => void;
  dataSource?: DataSourceInstanceSettings;
}

interface SaveAddRecordedQuery {
  name: string;
  description: string;
  aggregationType: string;
  interval: string;
  range: string;
  isCount: boolean;
}

export const CreateRecordedQuery = ({ query, queries, dataSource }: Props) => {
  const [isOpen, setIsOpen] = useState(false);
  const [alertSeverity, setAlertSeverity] = useState<AlertVariant>('info');
  const [alertMessage, setAlertMessage] = useState('');

  const toRecordedQuery = (data: SaveAddRecordedQuery): RecordedQuery => {
    return {
      target_ref_id: query ? query.refId : '',
      name: data.name,
      description: data.description,
      interval: intervalToSeconds(data.interval),
      range: intervalToSeconds(data.range),
      count: data.isCount,
      active: true,
      queries: queries?.map((q) => {
        const idQ = q as IdDataQuery;
        const dataSourceName = q.datasource || dataSource?.name;
        const ds = getDataSourceSrv().getInstanceSettings(dataSourceName);
        idQ.datasourceId = ds ? ds.id : -1;
        return idQ;
      }),
    } as RecordedQuery;
  };

  const showRecordQuery = config?.licenseInfo?.hasLicense && config?.featureToggles?.recordedQueries;
  if (!showRecordQuery) {
    return null;
  }

  const closeModal = () => {
    setAlertMessage('');
    setAlertSeverity('info');
    setIsOpen(false);
  };

  return (
    <>
      <QueryOperationAction
        title="Create Recorded Query"
        icon="history"
        onClick={() => {
          setIsOpen(true);
        }}
      />
      <Modal
        isOpen={isOpen}
        title={'Create Record Query'}
        onDismiss={closeModal}
        className={css`
          width: 500px;
        `}
      >
        <Form
          defaultValues={{
            name: '',
            description: '',
            aggregationType: '',
            interval: '',
            range: '',
            isCount: false,
          }}
          onSubmit={async (data) => {
            saveRecordedQuery(toRecordedQuery(data))
              .then(() => {
                closeModal();
                appEvents.emit(AppEvents.alertSuccess, [`Created Recorded Query: ${data.name}`]);
              })
              .catch((error) => {
                appEvents.emit(AppEvents.alertError, [error.data.message]);
              });
          }}
          validateOn="all"
        >
          {({ register, errors, getValues }) => (
            <>
              <Field label="Name" invalid={!!errors.name} error={errors.name?.message}>
                <Input
                  {...register('name', { required: { value: true, message: 'Name is required' } })}
                  aria-label="Add recorded query name"
                  autoFocus
                />
              </Field>
              <Field label="Description">
                <Input {...register('description')} aria-label="Add recorded query description" />
              </Field>
              <Field label="Interval" invalid={!!errors.interval} error={errors.interval?.message}>
                <Input
                  {...register('interval', { validate: validateTimeRange(1) })}
                  aria-label="Add recorded query interval"
                />
              </Field>
              <Field label="Range" invalid={!!errors.range} error={errors.range?.message}>
                <Input
                  {...register('range', { validate: validateTimeRange(0) })}
                  aria-label="Add recorded query range"
                />
              </Field>
              <Field label="Count Query Results">
                <Switch {...register('isCount')} aria-label="Record the count of the returned value" />
              </Field>
              <Alert
                severity={alertSeverity}
                title={'Test Recorded Query'}
                buttonContent={
                  <div aria-label="Test the recorded query">
                    <Icon name="sync" />
                  </div>
                }
                onRemove={() => {
                  testRecordedQuery(toRecordedQuery(getValues()))
                    .then((value) => {
                      setAlertSeverity('info');
                      setAlertMessage(`Result: ${value.data.message}`);
                    })
                    .catch((error) => {
                      setAlertSeverity('error');
                      setAlertMessage(error.data.message);
                    });
                }}
              >
                <>{alertMessage}</>
              </Alert>
              <Modal.ButtonRow>
                <Button
                  type="button"
                  variant="secondary"
                  onClick={closeModal}
                  fill="outline"
                  aria-label="Cancel recording query button"
                >
                  Cancel
                </Button>
                <Button type="submit" aria-label="Start recording query button">
                  Start recording query
                </Button>
              </Modal.ButtonRow>
            </>
          )}
        </Form>
      </Modal>
    </>
  );
};

function validateTimeRange(min: number): (val: string) => any {
  return (val: string) => {
    try {
      const time = describeInterval(val);
      if (time.count < min) {
        return `Time must be greater than or equal to ${min}`;
      }
      return true;
    } catch (e) {
      return e.message;
    }
  };
}
